/** 
 *   Test del Horno del IAR
 *
 *   (c) Copyright 2005 Federico Bareilles <fede@iar.unlp.edu.ar>,
 *   Instituto Argentino de Radio Astronomia (IAR).
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *     
 *   The author does NOT admit liability nor provide warranty for any
 *   of this software. This material is provided "AS-IS" in the hope
 *   that it may be useful for others.
 *
 **/

#define _GNU_SOURCE

int NTHR = 0;

#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>
#include <sys/time.h>
#include "thermometer.h"
#include "my_stdio.h"
#include "dwt.h"
#include "logger_novus.h"

struct thermometers {
	char *name;
	char *device;
	int fd;
	int num; /* number of values read */
	int (*open)(char *name);
	int (*close)(int dev);
	int (*get)(int dev, double *val);
};


struct logger_input ther_nv[] = {
	{ 1, PT100, 1},
	{ 2, PT100, 1},
	{ 3, PT100, 1},
/*	{ 4, PT100, 1},
	{ 5, PT100, 1},
	{ 6, PT100, 1},
	{ 7, PT100, 1},
	{ 8, PT100, 1},
*/
};
int ther_nv_length = sizeof(ther_nv)/sizeof(ther_nv[0]);


#define PRINT_REG(time_secs, temp) {\
        int __i;\
        printf("%10.2f", time_secs);\
        for(__i=0;__i<NTHR;__i++) printf(" %5.1f", temp[__i]);\
        printf("\n");\
        }
        /* WARNING: %10.2f anda 115 dias*/

#define DIFTV(t1, t0) (\
        (double)(t1.tv_sec - t0.tv_sec) + \
        (double)(t1.tv_usec - t0.tv_usec)/1000000.0)

/*
map t[] ={
	{ "IN_TEMP",   9, 2, GET_temp, 'F', "Indoor temperature",  DEF_TM },
	{ "OUT_TEMP", 12, 2, GET_temp, 'F', "Outdoor temperature", DEF_TM },
};
*/

int if_dif_temp(double *temp0, double *temp1)
{
	int i;
	int ret = 0;

	for(i = 0; i < NTHR; i++) {
		if (temp0[i] != temp1[i]) {
			ret = 1;
			break;
		}
	}

	return ret;
}


int dwt_get_temperatures(int fd_dwt, double *temp)
{
	char *buf = NULL;
	float *dwt_temp;

	if ( wake_recv(fd_dwt) == 0) {
		fprintf(stderr,"ERROR: wake_recv dwt\n");
		return -1;			
	}
	buf = GET_loop(fd_dwt);
	if(buf){
		dwt_temp = GET_temp(buf,  9, 2, DEF_TM);
		*(temp+1) = (double) *dwt_temp;
		dwt_temp = GET_temp(buf, 12, 2, DEF_TM);
		*(temp) = (double) *dwt_temp;
		free(buf);
	}

	return 0;
}


int logger_get(int dev, double *temp)
{
	int i;
	int ret;

	ret = logger_adq(dev, ther_nv, ther_nv_length);
	for(i=0;i<ther_nv_length && ret;i++) {
		*(temp+i) = ther_nv[i].val;
	}

	return ret;
}


int get_temperatures(struct thermometers *t, double *temp)
{
	int i;
	int ret = 0;
	int tp = 0;

	for(i=0; t[i].name != NULL && ret == 0;i++) {
		if (t[i].fd > 0) {
			ret = t[i].get(t[i].fd, temp+tp);
			tp+= t[i].num;
		}
	}

	return ret;
}

const char *help_text[] ={
#       ifdef VERBOSE_MODE
        "  -v\t\t - Verbose mode (incremental)",
#       endif
	"  -h\t\t - this Help",
	"  -a\t\t - print All temperatures",
//	"  -d device\t - thermometer Device (def "THERMOMETER_DEVICE")",
//	"  -l device\t - Logger device (def "LOGGER_NOVUS_DEVICE")",
	"  -s time\t - Saturation time to stop (sec)",
	"  -w time\t - Wait until saturation time is apply (sec)",
	"  -i time\t - Time between measurements (usec)",
	""
};
	

void usage(char *prog)
{
	int i=0;

	fprintf(stderr, 
		"Use %s [-vha] [-d device (def: %s)] [-l device (def: %s)] [-s time] [-w time]\n", 
		prog, THERMOMETER_DEVICE, LOGGER_NOVUS_DEVICE);
	while((char) *help_text[i] != 0)
                fprintf(stderr,"%s.\n",help_text[i++]);
        fprintf(stderr,"\n");
};


int open_thermometers(struct thermometers *t)
{
	int i;

	NTHR = 0;
	for(i=0;t[i].name != NULL;i++) {
		if ((t[i].fd = t[i].open(t[i].device)) < 0) {
			fprintf(stderr,"%s: %s,", t[i].name, t[i].device);
			perror("cannot open serial port");
			return -1;
		}
		NTHR += t[i].num;
	}

	return 0;
}


int close_thermometers(struct thermometers *t)
{
	int i;

	for(i=0;t[i].name != NULL;i++) {
		t[i].close(t[i].fd);
	}

	return 0;
}


struct thermometers thr[] = {
//	{ "ser", THERMOMETER_DEVICE, -1, 1, 
//	  thermometer_open, thermometer_close, thermometer_get },
//	{ "dwt", DWT_PORT, -1, 2,
//	  open_dwt, close, dwt_get_temperatures },
	{ "logger", LOGGER_NOVUS_DEVICE, -1, 
	  sizeof(ther_nv)/sizeof(ther_nv[0]),
	  logger_init, logger_close, logger_get},
	{NULL}
};



int main(int argc, char * const argv[])
{
	int i, j;
	int op;
	double *temp0, *temp1;
	double now_t0;
	//char *device = THERMOMETER_DEVICE;
	//char *logger_dev = LOGGER_NOVUS_DEVICE;
	struct timeval t0;
	struct timeval last;
	struct timeval now;
	struct timeval prev;
	struct timeval int_sleep;
	int done = 0;
	int print;
	int skip = 0;
	int stop_flag = 0;
	int print_all_flag = 0;
	time_t wait_until_stop = 0;
	time_t saturation_time = 0;
	unsigned long interval = 0;
	char title0[256];
	int tl0;
	char title1[256];
	int tl1;
	int count;

#ifdef VERBOSE_MODE
	verbose_flag = PERR;
#endif
	while  ((op = getopt(argc, argv, "vhd:s:w:i:a")) != EOF)
		switch( op ){
#ifdef VERBOSE_MODE
		case 'v':
			verbose_flag = (verbose_flag << 1) | 1;
			break;
#endif
/*
		case 'd':
			device = optarg;
			break;
		case 'l':
			logger_dev = optarg;
			break;
*/
		case 's':
			stop_flag = 1;
			saturation_time = (time_t) atol(optarg);
			break;
		case 'w':
			stop_flag = 1;
			wait_until_stop = (time_t) atol(optarg);
			break;
		case 'i':
			interval = (unsigned long) atol(optarg);
			break;
		case 'a':
			print_all_flag = 1;
			break;
		case 'h':
		default:
			usage(argv[0]);
			exit(-1);
		}

	if (open_thermometers(thr) != 0) {
		fprintf(stderr, "Error\n");
		return -1;
	}
	temp0 = (double *) malloc(sizeof(double) * NTHR); 
	temp1 = (double *) malloc(sizeof(double) * NTHR); 
	if (get_temperatures(thr, temp0) != 0) {
		fprintf(stderr, "Error\n");
		return -1;
	}
	gettimeofday(&t0, NULL);
	memcpy(&last, &t0, sizeof(struct timeval));
	printf("# DATE = %ld.%6ld\n", t0.tv_sec, t0.tv_usec);
	if ( stop_flag ) {
		printf("# s = %ld\n", saturation_time );
		printf("# w = %ld\n", wait_until_stop );
	}
	if (print_all_flag) {
		printf("# a = on\n");
	}
	tl0 = sprintf(title0, "#          +");
	tl1 = sprintf(title1, "# Time     ");
	for (i=0; thr[i].name != NULL;i++) {
		char format[16];
		sprintf(format,"%%-%ds+", (6 * thr[i].num) -1 );
		tl0 += sprintf(title0+tl0, format, thr[i].name);
		for(j=0;j<thr[i].num;j++) {
			tl1 += sprintf(title1+tl1,"%5d ", j+1);
		}
	} 
	printf("%s\n", title0);
	printf("%s\n", title1);

	PRINT_REG(0.0, temp0);
	fflush(stdout);
	gettimeofday(&now, NULL);
	count = 0;
	while(!done) {
		if(interval >= 100000) {
			unsigned long time;
			gettimeofday(&int_sleep, NULL);
			time = interval - 10000 - (unsigned long) 
				(DIFTV(int_sleep, t0) * 1000000.0 - 
				 (double) count * (double)interval);
			//printf("sleeping: %ld usec\n", time);
			if(time < 10000000)
				usleep(time);
			//usleep(interval - DIFTV(int_sleep, now) * 1000000.0 
			//     - 10000);
		}
		gettimeofday(&now, NULL);
		if (get_temperatures(thr, temp1) != 0) {
			fprintf(stderr, "Error\n");
			return -1;
		}
		now_t0 = DIFTV(now, t0);
		print = 0;
		if ( if_dif_temp(temp1, temp0) || print_all_flag ) {
			if (skip > 0) {
				PRINT_REG(DIFTV(prev, t0), temp0);
			}
			//printf("skip: %d ", skip);
			memcpy(&last, &now, sizeof(struct timeval));
			memcpy(temp0, temp1, sizeof(double) * NTHR);
			PRINT_REG(now_t0, temp1);
			fflush(stdout);
			print = 1;
			skip = 0;
		} 
		else {
			memcpy(&prev, &now, sizeof(struct timeval));
			skip++;
		}
		if (stop_flag &&  
		    (time_t) now_t0 >= wait_until_stop &&
		    (time_t) DIFTV(now, last) >= saturation_time   ) {
			done = 1;
			if (!print) PRINT_REG(now_t0, temp1);
		}
		count++;
	}
	fflush(stdout);
	close_thermometers(thr);
	fprintf(stderr, "%c", 0x7);
	//free(temp0);
	//free(temp1);
	
	return 0;
}

