
#include <stdio.h>
#include <time.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <ctype.h>
#include "dwt.h"


void usage(char *s)
{
	if (s) 
		printf("dwt: %s\n\n", s);
	else 
		printf("Davis Weather Tool - dwt 1.0\n\n");

	printf("usage: dwt COMMAND PARAMS [options]

   COMMANDS      PARAMS
        FRMT    +FORMAT    Output formatted observation fields as defined in format string
        TAGS               List observation tags used for format string
        LIST               List all observation fields
        GETTIME            Display time of receiver
        SETTIME            Set time of receiver to that of PC
        TEST               Test communications
        WRD                Display weather station type
        VER                Display firmware version
        RXCHECK            Display receiver statistics
     
 Options:
       -s tty     Specify alternate serial port
       -r num     Repeat FRMT command 'num' times
       -d delay   Delay after FRMT command, valid if -r option used\n\n");

	exit(-1);
}



int main(int argc, char **argv)
{
	int	fd, c;
	ushort	cmd, num = 1, delay = 0;
	char	*tty = NULL, *frmt = NULL;

	if (argc < 2) 
		usage(NULL);

	if (!valid_cmd(argv[1]))
		usage("Invalid command");

	cmd = get_cmd(argv[1]);

	if (cmd == 1) {
		if (argc < 3)
			usage(NULL);

		if  (argv[2][0] != '+')
			usage("Invalid format specifier");

		frmt = argv[2]+1;
	}

	opterr=0;

	while ( (c = getopt( argc, argv, "s:r:d:")) != EOF)
		switch(c)
                {
                        case 's' :    
				tty = strdup(optarg);
                                break;
                        case 'r' :
				num = atoi(optarg);
                                break;
			case 'd'
				:
				delay = atoi(optarg);
				break;
                        default :
                                opterr++;
                                break;                                
                }                                                     

	if (opterr) 
		usage("Invalid options");

	if ((num>1  || delay>0) && (cmd != 1))
		usage("Invalid options - only valid for FRMT command");

	if (tty == NULL) 
		tty = DWT_PORT;

	if ((fd = open_dwt(tty)) == -1) {
		fprintf(stderr,"dwt: Cannot open serial port");
		exit(-1);
	}

	if (cmd  != 2)
		wake_recv(fd);

	switch(cmd)
	{
		case 1 :	
				CMD_frmt(fd, frmt, num, delay);
				break;

		case 2 :	CMD_tags();
				
				break;

		case 3 :
				CMD_list(fd);
				break;

		case 4 :
				CMD_gettime(fd);
				break;

		case 5 :
				CMD_settime(fd);
				break;

		case 6 :
				CMD_test(fd);
				break;

		case 7 :
				CMD_wrd(fd);
				break;

		case 8 :
				CMD_ver(fd);
				break;

		case 9 :
				CMD_rxcheck(fd);
				break;
		default :
				break;
	}
	close(fd);
	exit(0);
}
