/** 
 *   Thermometer logger novus
 *
 *   (c) Copyright 2005 Federico Bareilles <fede@iar.unlp.edu.ar>,
 *   Instituto Argentino de Radio Astronomia (IAR).
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *     
 *   The author does NOT admit liability nor provide warranty for any
 *   of this software. This material is provided "AS-IS" in the hope
 *   that it may be useful for others.
 *
 **/


#define _GNU_SOURCE

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "modbus.h"
#include "logger_novus.h"


int logger_init(char *device)
{
	return modbus_open(device);
}


int logger_close(int dev)
{
	modbus_close(dev);

	return 0;
}


static double logger_scale(int type, int val)
{
	double result;

	switch(type) {
	case PT100:
		result = (double) val / 10.0;
		break;
	case THERMOCOUPLE_J:
	case THERMOCOUPLE_K:
	case THERMOCOUPLE_T:
	case THERMOCOUPLE_E:
	case THERMOCOUPLE_N:
	case THERMOCOUPLE_R:
	case THERMOCOUPLE_S:
	case THERMOCOUPLE_B:
		result = (double) val;
		break;
	default:
		printf("unknown input devece 0x%x\n", type);
		result = (double) val;
		break;
	}

	return result;
}


int logger_adq(int fd, struct logger_input *tn, int length)
{
	int result;
	int len, i;
	unsigned char pk[MODBUS_PACKET_LENGTH];

	memset(pk, 0, 8);
	len = modbus_make_pk(pk, LOGGER_SLAVE, 3, 0, length);
	result = modbus_write_pk(fd, pk, len);
	if (result > 0) {
		result = 0;
		len = modbus_read_pk(fd, pk, 3 + length * 2 + 2);
		if (len > 0 ) { 
			for(i=0;i<length;i++) {
				if (tn[i].active) {
					tn[i].val = 
						logger_scale(tn[i].type, 
							     pk[3+(2*i)+1] | 
							     pk[3+(2*i)+2]<<8);
					result++;
				}
			}
		}else{
			result = len;
		}
	}

	return result;
}

