/** 
 *   Thermometer logger novus
 *
 *   (c) Copyright 2005 Federico Bareilles <fede@iar.unlp.edu.ar>,
 *   Instituto Argentino de Radio Astronomia (IAR).
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *     
 *   The author does NOT admit liability nor provide warranty for any
 *   of this software. This material is provided "AS-IS" in the hope
 *   that it may be useful for others.
 *
 **/

//#include <modbus.h>
#include "logger_novus.h"

//Mb_verbose = 1;

int logger_init(char *mb_device, Mbm_trame *tr)
{
	tr->timeout = 1000;
	tr->device = Mb_open_device(mb_device, 9600, 0, 8, 1);

	return tr->device;
}


int logger_close(int dev)
{
	Mb_close_device(dev);

	return 0;
}


static double logger_scale(int type, int val)
{
	double result;

	switch(type) {
	case PT100:
		result = (double) val / 10.0;
		break;
	case THERMOCOUPLE_J:
	case THERMOCOUPLE_K:
	case THERMOCOUPLE_T:
	case THERMOCOUPLE_E:
	case THERMOCOUPLE_N:
	case THERMOCOUPLE_R:
	case THERMOCOUPLE_S:
	case THERMOCOUPLE_B:
		result = (double) val;
		break;
	default:
		printf("unknown input devece 0x%x\n", type);
		result = (double) val;
		break;
	}

	return result;
}

int logger_adq(Mbm_trame *tr, struct logger_input *tn)
{
	int data_out[256];
	int result;
	int i;

	tr->function = 3;
	tr->timeout = 1000;

	result = Mb_master(*tr, NULL, data_out, NULL, NULL);
	if (result<0) {
		if (result==-1)	printf("Function, parameter error.");
		if (result==-2)	printf("CRC error.");
		if (result==-3) printf("Time out.");
		if (result==-4) printf("Error : bad answer.");
	} else {
		for(i=0;i<tr->length;i++) {
			if (tn[i].active) {
				tn[i].val = 
					logger_scale(tn[i].type, 
						    data_out[i]);
			}

		}
	}

	return 0;
}



int logger_dig_in(Mbm_trame *tr, struct logger_input *tn)
{
	int data_in[256];
	int data_out[256];
	int result;
	//int i;

	tr->function = 2;
	tr->timeout = 1000;
	//for(i=0x7e;i<128;i++){
	tr->address = 100;
	data_in[0] = 1;
	result = Mb_master(*tr, data_in, data_out, NULL, NULL);
	if (result<0) {
		if (result==-1)	printf("Function, parameter error.");
		if (result==-2)	printf("CRC error.");
		if (result==-3) printf("Time out.");
		if (result==-4) printf("Error : bad answer.");
	} else {
		/*
		for(i=0;i<tr->length;i++) {
			if (tn[i].active) {
				tn[i].val = 
					logger_scale(tn[i].type, 
						    data_out[i]);
			}

		}
		*/
		printf("\n\ndata_out: 0x%x (%d)\n", data_out[0], data_out[1]);
		return 0;
	}
//}
	return 0;
}


int logger_dig_out(Mbm_trame *tr, struct logger_input *tn)
{
	int data_in[256];
	int data_out[256];
	int result;
	int i;

	tr->function = 5;
	tr->timeout = 1000;
	for(i=0;i<2;i++) {
	tr->address = i;
	data_in[0] = 0x0000;
	result = Mb_master(*tr, data_in, data_out, NULL, NULL);
	if (result<0) {
		if (result==-1)	printf("Function, parameter error.");
		if (result==-2)	printf("CRC error.");
		if (result==-3) printf("Time out.");
		if (result==-4) printf("Error : bad answer.");
	} else {
		/*
		for(i=0;i<tr->length;i++) {
			if (tn[i].active) {
				tn[i].val = 
					logger_scale(tn[i].type, 
						    data_out[i]);
			}

		}
		*/
		printf("data_out: 0x%x (%d)\n", data_out[0], data_out[0]);
	}
	}
	return 0;
}

